@@ -1129,6 +1129,15 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             schema.CreateIndex(idx), "CREATE CLUSTERED INDEX foo ON test (id)"
         )
 
+    def test_index_where(self):
+        metadata = MetaData()
+        tbl = Table("test", metadata, Column("data", Integer))
+        idx = Index("test_idx_data_1", tbl.c.data, mssql_where=tbl.c.data > 1)
+        self.assert_compile(
+            schema.CreateIndex(idx),
+            "CREATE INDEX test_idx_data_1 ON test (data) WHERE data > 1"
+        )
+
     def test_index_ordering(self):
         metadata = MetaData()
         tbl = Table(
