@@ -28,6 +28,7 @@ from sqlalchemy.dialects.postgresql import TSRANGE
 from sqlalchemy.engine import reflection
 from sqlalchemy.sql.schema import CheckConstraint
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing import mock
 from sqlalchemy.testing.assertions import assert_raises
 from sqlalchemy.testing.assertions import AssertsExecutionResults
 from sqlalchemy.testing.assertions import eq_
@@ -96,6 +97,47 @@ class ForeignTableReflectionTest(fixtures.TablesTest, AssertsExecutionResults):
             eq_(names, ["testtable"])
 
 
+class PartitionedReflectionTest(fixtures.TablesTest, AssertsExecutionResults):
+    # partitioned table reflection, issue #4237
+
+    __only_on__ = "postgresql >= 10"
+    __backend__ = True
+
+    @classmethod
+    def define_tables(cls, metadata):
+        # the actual function isn't reflected yet
+        dv = Table(
+            "data_values",
+            metadata,
+            Column("modulus", Integer, nullable=False),
+            Column("data", String(30)),
+            postgresql_partition_by="range(modulus)",
+        )
+
+        # looks like this is reflected prior to #4237
+        sa.event.listen(
+            dv,
+            "after_create",
+            sa.DDL(
+                "CREATE TABLE data_values_4_10 PARTITION OF data_values "
+                "FOR VALUES FROM (4) TO (10)"
+            ),
+        )
+
+    def test_get_tablenames(self):
+        assert {"data_values", "data_values_4_10"}.issubset(
+            inspect(testing.db).get_table_names()
+        )
+
+    def test_reflect_cols(self):
+        cols = inspect(testing.db).get_columns("data_values")
+        eq_([c["name"] for c in cols], ["modulus", "data"])
+
+    def test_reflect_cols_from_partition(self):
+        cols = inspect(testing.db).get_columns("data_values_4_10")
+        eq_([c["name"] for c in cols], ["modulus", "data"])
+
+
 class MaterializedViewReflectionTest(
     fixtures.TablesTest, AssertsExecutionResults
 ):
@@ -865,6 +907,83 @@ class ReflectionTest(fixtures.TestBase):
             ],
         )
 
+    @testing.fails_if("postgresql < 8.3", "index ordering not supported")
+    @testing.provide_metadata
+    def test_index_reflection_with_sorting(self):
+        """reflect indexes with sorting options set"""
+
+        t1 = Table(
+            "party",
+            self.metadata,
+            Column("id", String(10), nullable=False),
+            Column("name", String(20)),
+            Column("aname", String(20)),
+        )
+
+        with testing.db.connect() as conn:
+
+            t1.create(conn)
+
+            # check ASC, DESC options alone
+            conn.execute(
+                """
+                create index idx1 on party
+                    (id, name ASC, aname DESC)
+            """
+            )
+
+            # check DESC w/ NULLS options
+            conn.execute(
+                """
+              create index idx2 on party
+                    (name DESC NULLS FIRST, aname DESC NULLS LAST)
+            """
+            )
+
+            # check ASC w/ NULLS options
+            conn.execute(
+                """
+              create index idx3 on party
+                    (name ASC NULLS FIRST, aname ASC NULLS LAST)
+            """
+            )
+
+        # reflect data
+        with testing.db.connect() as conn:
+            m2 = MetaData(conn)
+            t2 = Table("party", m2, autoload=True)
+
+        eq_(len(t2.indexes), 3)
+
+        # Make sure indexes are in the order we expect them in
+        r1, r2, r3 = sorted(t2.indexes, key=lambda idx: idx.name)
+
+        eq_(r1.name, "idx1")
+        eq_(r2.name, "idx2")
+        eq_(r3.name, "idx3")
+
+        # "ASC NULLS LAST" is implicit default for indexes,
+        # and "NULLS FIRST" is implicit default for "DESC".
+        # (https://www.postgresql.org/docs/11/indexes-ordering.html)
+
+        def compile_exprs(exprs):
+            return list(map(str, exprs))
+
+        eq_(
+            compile_exprs([t2.c.id, t2.c.name, t2.c.aname.desc()]),
+            compile_exprs(r1.expressions),
+        )
+
+        eq_(
+            compile_exprs([t2.c.name.desc(), t2.c.aname.desc().nullslast()]),
+            compile_exprs(r2.expressions),
+        )
+
+        eq_(
+            compile_exprs([t2.c.name.nullsfirst(), t2.c.aname]),
+            compile_exprs(r3.expressions),
+        )
+
     @testing.provide_metadata
     def test_index_reflection_modified(self):
         """reflect indexes when a column name has changed - PG 9
@@ -1248,6 +1367,33 @@ class ReflectionTest(fixtures.TestBase):
             ],
         )
 
+    @testing.provide_metadata
+    def test_inspect_enum_empty(self):
+        enum_type = postgresql.ENUM(name="empty", metadata=self.metadata)
+        enum_type.create(testing.db)
+        inspector = reflection.Inspector.from_engine(testing.db)
+
+        eq_(
+            inspector.get_enums(),
+            [
+                {
+                    "visible": True,
+                    "labels": [],
+                    "name": "empty",
+                    "schema": "public",
+                }
+            ],
+        )
+
+    @testing.provide_metadata
+    def test_inspect_enum_empty_from_table(self):
+        Table(
+            "t", self.metadata, Column("x", postgresql.ENUM(name="empty"))
+        ).create(testing.db)
+
+        t = Table("t", MetaData(testing.db), autoload_with=testing.db)
+        eq_(t.c.x.type.enums, [])
+
     @testing.provide_metadata
     @testing.only_on("postgresql >= 8.5")
     def test_reflection_with_unique_constraint(self):
@@ -1383,6 +1529,20 @@ class ReflectionTest(fixtures.TestBase):
             },
         )
 
+    def test_reflect_check_warning(self):
+        conn = mock.Mock(
+            execute=lambda *arg, **kw: mock.Mock(
+                fetchall=lambda: [("some name", "NOTCHECK foobar")]
+            )
+        )
+        with mock.patch.object(
+            testing.db.dialect, "get_table_oid", lambda *arg, **kw: 1
+        ):
+            with testing.expect_warnings(
+                "Could not parse CHECK constraint text: 'NOTCHECK foobar'"
+            ):
+                testing.db.dialect.get_check_constraints(conn, "foo")
+
 
 class CustomTypeReflectionTest(fixtures.TestBase):
     class CustomType(object):
