@@ -1,34 +1,47 @@
-from sqlalchemy.testing import assert_raises, assert_raises_message
-from sqlalchemy.orm import util as orm_util
 from sqlalchemy import Column
-from sqlalchemy import util
+from sqlalchemy import inspect
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
 from sqlalchemy import Table
-from sqlalchemy.orm import aliased, with_polymorphic, synonym
-from sqlalchemy.orm import mapper, create_session, Session
-from sqlalchemy.testing import fixtures
-from test.orm import _fixtures
-from sqlalchemy.testing import eq_, is_
-from sqlalchemy.orm.path_registry import PathRegistry, RootRegistry
-from sqlalchemy import inspect
-from sqlalchemy.ext.hybrid import hybrid_property, hybrid_method
+from sqlalchemy.ext.hybrid import hybrid_method
+from sqlalchemy.ext.hybrid import hybrid_property
+from sqlalchemy.orm import aliased
+from sqlalchemy.orm import create_session
+from sqlalchemy.orm import mapper
+from sqlalchemy.orm import Session
+from sqlalchemy.orm import synonym
+from sqlalchemy.orm import util as orm_util
+from sqlalchemy.orm import with_polymorphic
+from sqlalchemy.orm.path_registry import PathRegistry
+from sqlalchemy.orm.path_registry import RootRegistry
+from sqlalchemy.testing import assert_raises
 from sqlalchemy.testing import AssertsCompiledSQL
+from sqlalchemy.testing import eq_
+from sqlalchemy.testing import fixtures
+from sqlalchemy.testing import is_
+from sqlalchemy.util import compat
+from test.orm import _fixtures
+from .inheritance import _poly_fixtures
+
 
 class AliasedClassTest(fixtures.TestBase, AssertsCompiledSQL):
-    __dialect__ = 'default'
+    __dialect__ = "default"
 
     def _fixture(self, cls, properties={}):
-        table = Table('point', MetaData(),
-                    Column('id', Integer(), primary_key=True),
-                    Column('x', Integer),
-                    Column('y', Integer))
+        table = Table(
+            "point",
+            MetaData(),
+            Column("id", Integer(), primary_key=True),
+            Column("x", Integer),
+            Column("y", Integer),
+        )
         mapper(cls, table, properties=properties)
         return table
 
     def test_simple(self):
         class Point(object):
             pass
+
         table = self._fixture(Point)
 
         alias = aliased(Point)
@@ -43,6 +56,7 @@ class AliasedClassTest(fixtures.TestBase, AssertsCompiledSQL):
     def test_not_instantiatable(self):
         class Point(object):
             pass
+
         table = self._fixture(Point)
         alias = aliased(Point)
 
@@ -58,12 +72,7 @@ class AliasedClassTest(fixtures.TestBase, AssertsCompiledSQL):
 
         assert Point.zero
 
-        # TODO: I don't quite understand this
-        # still
-        if util.py2k:
-            assert not getattr(alias, 'zero')
-        else:
-            assert getattr(alias, 'zero')
+        assert getattr(alias, "zero")
 
     def test_classmethod(self):
         class Point(object):
@@ -93,20 +102,22 @@ class AliasedClassTest(fixtures.TestBase, AssertsCompiledSQL):
         assert Point.max_x is alias.max_x
 
     def test_descriptors(self):
-
         class descriptor(object):
             def __init__(self, fn):
                 self.fn = fn
+
             def __get__(self, obj, owner):
                 if obj is not None:
                     return self.fn(obj, obj)
                 else:
                     return self
+
             def method(self):
-                return 'method'
+                return "method"
 
         class Point(object):
             center = (0, 0)
+
             @descriptor
             def thing(self, arg):
                 return arg.center
@@ -116,13 +127,14 @@ class AliasedClassTest(fixtures.TestBase, AssertsCompiledSQL):
 
         assert Point.thing != (0, 0)
         assert Point().thing == (0, 0)
-        assert Point.thing.method() == 'method'
+        assert Point.thing.method() == "method"
 
         assert alias.thing != (0, 0)
-        assert alias.thing.method() == 'method'
+        assert alias.thing.method() == "method"
 
     def _assert_has_table(self, expr, table):
         from sqlalchemy import Column  # override testlib's override
+
         for child in expr.get_children():
             if isinstance(child, Column):
                 assert child.table is table
@@ -144,7 +156,7 @@ class AliasedClassTest(fixtures.TestBase, AssertsCompiledSQL):
             sess.query(alias).filter(alias.left_of(Point)),
             "SELECT point_1.id AS point_1_id, point_1.x AS point_1_x, "
             "point_1.y AS point_1_y FROM point AS point_1, point "
-            "WHERE point_1.x < point.x"
+            "WHERE point_1.x < point.x",
         )
 
     def test_hybrid_descriptor_two(self):
@@ -170,7 +182,7 @@ class AliasedClassTest(fixtures.TestBase, AssertsCompiledSQL):
             sess.query(alias).filter(alias.double_x > Point.x),
             "SELECT point_1.id AS point_1_id, point_1.x AS point_1_x, "
             "point_1.y AS point_1_y FROM point AS point_1, point "
-            "WHERE point_1.x * :x_1 > point.x"
+            "WHERE point_1.x * :x_1 > point.x",
         )
 
     def test_hybrid_descriptor_three(self):
@@ -197,10 +209,7 @@ class AliasedClassTest(fixtures.TestBase, AssertsCompiledSQL):
         eq_(str(alias.x + 1), "point_1.x + :x_1")
         eq_(str(alias.x_alone + 1), "point_1.x + :x_1")
 
-        is_(
-            Point.x_alone.__clause_element__(),
-            Point.x.__clause_element__()
-        )
+        is_(Point.x_alone.__clause_element__(), Point.x.__clause_element__())
 
         eq_(str(alias.x_alone == alias.x), "point_1.x = point_1.x")
 
@@ -213,7 +222,7 @@ class AliasedClassTest(fixtures.TestBase, AssertsCompiledSQL):
             sess.query(alias).filter(alias.x_alone > Point.x),
             "SELECT point_1.id AS point_1_id, point_1.x AS point_1_x, "
             "point_1.y AS point_1_y FROM point AS point_1, point "
-            "WHERE point_1.x > point.x"
+            "WHERE point_1.x > point.x",
         )
 
     def test_proxy_descriptor_one(self):
@@ -221,9 +230,7 @@ class AliasedClassTest(fixtures.TestBase, AssertsCompiledSQL):
             def __init__(self, x, y):
                 self.x, self.y = x, y
 
-        self._fixture(Point, properties={
-            'x_syn': synonym("x")
-        })
+        self._fixture(Point, properties={"x_syn": synonym("x")})
         alias = aliased(Point)
 
         eq_(str(Point.x_syn), "Point.x_syn")
@@ -233,16 +240,114 @@ class AliasedClassTest(fixtures.TestBase, AssertsCompiledSQL):
         self.assert_compile(
             sess.query(alias.x_syn).filter(alias.x_syn > Point.x_syn),
             "SELECT point_1.x AS point_1_x FROM point AS point_1, point "
-            "WHERE point_1.x > point.x"
+            "WHERE point_1.x > point.x",
+        )
+
+    def test_meta_getattr_one(self):
+        class MetaPoint(type):
+            def __getattr__(cls, key):
+                if key == "x_syn":
+                    return cls.x
+                raise AttributeError(key)
+
+        class Point(compat.with_metaclass(MetaPoint)):
+            pass
+
+        self._fixture(Point)
+        alias = aliased(Point)
+
+        eq_(str(Point.x_syn), "Point.x")
+        eq_(str(alias.x_syn), "AliasedClass_Point.x")
+
+        # from __clause_element__() perspective, Point.x_syn
+        # and Point.x return the same thing, so that's good
+        eq_(str(Point.x.__clause_element__()), "point.x")
+        eq_(str(Point.x_syn.__clause_element__()), "point.x")
+
+        # same for the alias
+        eq_(str(alias.x + 1), "point_1.x + :x_1")
+        eq_(str(alias.x_syn + 1), "point_1.x + :x_1")
+
+        is_(Point.x_syn.__clause_element__(), Point.x.__clause_element__())
+
+        eq_(str(alias.x_syn == alias.x), "point_1.x = point_1.x")
+
+        a2 = aliased(Point)
+        eq_(str(a2.x_syn == alias.x), "point_1.x = point_2.x")
+
+        sess = Session()
+
+        self.assert_compile(
+            sess.query(alias).filter(alias.x_syn > Point.x),
+            "SELECT point_1.id AS point_1_id, point_1.x AS point_1_x, "
+            "point_1.y AS point_1_y FROM point AS point_1, point "
+            "WHERE point_1.x > point.x",
+        )
+
+    def test_meta_getattr_two(self):
+        class MetaPoint(type):
+            def __getattr__(cls, key):
+                if key == "double_x":
+                    return cls._impl_double_x
+                raise AttributeError(key)
+
+        class Point(compat.with_metaclass(MetaPoint)):
+            @hybrid_property
+            def _impl_double_x(self):
+                return self.x * 2
+
+        self._fixture(Point)
+        alias = aliased(Point)
+
+        eq_(str(Point.double_x), "Point._impl_double_x")
+        eq_(str(alias.double_x), "AliasedClass_Point._impl_double_x")
+        eq_(str(Point.double_x.__clause_element__()), "point.x * :x_1")
+        eq_(str(alias.double_x.__clause_element__()), "point_1.x * :x_1")
+
+        sess = Session()
+
+        self.assert_compile(
+            sess.query(alias).filter(alias.double_x > Point.x),
+            "SELECT point_1.id AS point_1_id, point_1.x AS point_1_x, "
+            "point_1.y AS point_1_y FROM point AS point_1, point "
+            "WHERE point_1.x * :x_1 > point.x",
+        )
+
+    def test_meta_getattr_three(self):
+        class MetaPoint(type):
+            def __getattr__(cls, key):
+                @hybrid_property
+                def double_x(me):
+                    return me.x * 2
+
+                if key == "double_x":
+                    return double_x.__get__(None, cls)
+                raise AttributeError(key)
+
+        class Point(compat.with_metaclass(MetaPoint)):
+            pass
+
+        self._fixture(Point)
+
+        alias = aliased(Point)
+
+        eq_(str(Point.double_x.__clause_element__()), "point.x * :x_1")
+        eq_(str(alias.double_x.__clause_element__()), "point_1.x * :x_1")
+
+        sess = Session()
+
+        self.assert_compile(
+            sess.query(alias).filter(alias.double_x > Point.x),
+            "SELECT point_1.id AS point_1_id, point_1.x AS point_1_x, "
+            "point_1.y AS point_1_y FROM point AS point_1, point "
+            "WHERE point_1.x * :x_1 > point.x",
         )
 
     def test_parententity_vs_parentmapper(self):
         class Point(object):
             pass
 
-        self._fixture(Point, properties={
-            'x_syn': synonym("x")
-        })
+        self._fixture(Point, properties={"x_syn": synonym("x")})
         pa = aliased(Point)
 
         is_(Point.x_syn._parententity, inspect(Point))
@@ -251,17 +356,21 @@ class AliasedClassTest(fixtures.TestBase, AssertsCompiledSQL):
         is_(Point.x._parentmapper, inspect(Point))
 
         is_(
-            Point.x_syn.__clause_element__()._annotations['parententity'],
-            inspect(Point))
+            Point.x_syn.__clause_element__()._annotations["parententity"],
+            inspect(Point),
+        )
         is_(
-            Point.x.__clause_element__()._annotations['parententity'],
-            inspect(Point))
+            Point.x.__clause_element__()._annotations["parententity"],
+            inspect(Point),
+        )
         is_(
-            Point.x_syn.__clause_element__()._annotations['parentmapper'],
-            inspect(Point))
+            Point.x_syn.__clause_element__()._annotations["parentmapper"],
+            inspect(Point),
+        )
         is_(
-            Point.x.__clause_element__()._annotations['parentmapper'],
-            inspect(Point))
+            Point.x.__clause_element__()._annotations["parentmapper"],
+            inspect(Point),
+        )
 
         pa = aliased(Point)
 
@@ -271,19 +380,20 @@ class AliasedClassTest(fixtures.TestBase, AssertsCompiledSQL):
         is_(pa.x._parentmapper, inspect(Point))
 
         is_(
-            pa.x_syn.__clause_element__()._annotations['parententity'],
-            inspect(pa)
+            pa.x_syn.__clause_element__()._annotations["parententity"],
+            inspect(pa),
         )
         is_(
-            pa.x.__clause_element__()._annotations['parententity'],
-            inspect(pa)
+            pa.x.__clause_element__()._annotations["parententity"], inspect(pa)
         )
         is_(
-            pa.x_syn.__clause_element__()._annotations['parentmapper'],
-            inspect(Point))
+            pa.x_syn.__clause_element__()._annotations["parentmapper"],
+            inspect(Point),
+        )
         is_(
-            pa.x.__clause_element__()._annotations['parentmapper'],
-            inspect(Point))
+            pa.x.__clause_element__()._annotations["parentmapper"],
+            inspect(Point),
+        )
 
 
 class IdentityKeyTest(_fixtures.FixtureTest):
@@ -295,9 +405,9 @@ class IdentityKeyTest(_fixtures.FixtureTest):
         mapper(User, users)
 
         key = orm_util.identity_key(User, [1])
-        eq_(key, (User, (1,)))
+        eq_(key, (User, (1,), None))
         key = orm_util.identity_key(User, ident=[1])
-        eq_(key, (User, (1,)))
+        eq_(key, (User, (1,), None))
 
     def test_identity_key_scalar(self):
         User, users = self.classes.User, self.tables.users
@@ -305,20 +415,20 @@ class IdentityKeyTest(_fixtures.FixtureTest):
         mapper(User, users)
 
         key = orm_util.identity_key(User, 1)
-        eq_(key, (User, (1,)))
+        eq_(key, (User, (1,), None))
         key = orm_util.identity_key(User, ident=1)
-        eq_(key, (User, (1,)))
+        eq_(key, (User, (1,), None))
 
     def test_identity_key_2(self):
         users, User = self.tables.users, self.classes.User
 
         mapper(User, users)
         s = create_session()
-        u = User(name='u1')
+        u = User(name="u1")
         s.add(u)
         s.flush()
         key = orm_util.identity_key(instance=u)
-        eq_(key, (User, (u.id,)))
+        eq_(key, (User, (u.id,), None))
 
     def test_identity_key_3(self):
         User, users = self.classes.User, self.tables.users
@@ -327,29 +437,32 @@ class IdentityKeyTest(_fixtures.FixtureTest):
 
         row = {users.c.id: 1, users.c.name: "Frank"}
         key = orm_util.identity_key(User, row=row)
-        eq_(key, (User, (1,)))
+        eq_(key, (User, (1,), None))
+
+    def test_identity_key_token(self):
+        User, users = self.classes.User, self.tables.users
+
+        mapper(User, users)
+
+        key = orm_util.identity_key(User, [1], identity_token="token")
+        eq_(key, (User, (1,), "token"))
+        key = orm_util.identity_key(User, ident=[1], identity_token="token")
+        eq_(key, (User, (1,), "token"))
 
 
 class PathRegistryTest(_fixtures.FixtureTest):
-    run_setup_mappers = 'once'
+    run_setup_mappers = "once"
     run_inserts = None
     run_deletes = None
 
-
     @classmethod
     def setup_mappers(cls):
         cls._setup_stock_mapping()
 
     def test_root_registry(self):
         umapper = inspect(self.classes.User)
-        is_(
-            RootRegistry()[umapper],
-            umapper._path_registry
-        )
-        eq_(
-            RootRegistry()[umapper],
-            PathRegistry.coerce((umapper,))
-        )
+        is_(RootRegistry()[umapper], umapper._path_registry)
+        eq_(RootRegistry()[umapper], PathRegistry.coerce((umapper,)))
 
     def test_expand(self):
         umapper = inspect(self.classes.User)
@@ -357,10 +470,17 @@ class PathRegistryTest(_fixtures.FixtureTest):
         path = PathRegistry.coerce((umapper,))
 
         eq_(
-            path[umapper.attrs.addresses][amapper]
-                [amapper.attrs.email_address],
-            PathRegistry.coerce((umapper, umapper.attrs.addresses,
-                                amapper, amapper.attrs.email_address))
+            path[umapper.attrs.addresses][amapper][
+                amapper.attrs.email_address
+            ],
+            PathRegistry.coerce(
+                (
+                    umapper,
+                    umapper.attrs.addresses,
+                    amapper,
+                    amapper.attrs.email_address,
+                )
+            ),
         )
 
     def test_entity_boolean(self):
@@ -383,24 +503,42 @@ class PathRegistryTest(_fixtures.FixtureTest):
     def test_indexed_entity(self):
         umapper = inspect(self.classes.User)
         amapper = inspect(self.classes.Address)
-        path = PathRegistry.coerce((umapper, umapper.attrs.addresses,
-                                amapper, amapper.attrs.email_address))
+        path = PathRegistry.coerce(
+            (
+                umapper,
+                umapper.attrs.addresses,
+                amapper,
+                amapper.attrs.email_address,
+            )
+        )
         is_(path[0], umapper)
         is_(path[2], amapper)
 
     def test_indexed_key(self):
         umapper = inspect(self.classes.User)
         amapper = inspect(self.classes.Address)
-        path = PathRegistry.coerce((umapper, umapper.attrs.addresses,
-                                amapper, amapper.attrs.email_address))
+        path = PathRegistry.coerce(
+            (
+                umapper,
+                umapper.attrs.addresses,
+                amapper,
+                amapper.attrs.email_address,
+            )
+        )
         eq_(path[1], umapper.attrs.addresses)
         eq_(path[3], amapper.attrs.email_address)
 
     def test_slice(self):
         umapper = inspect(self.classes.User)
         amapper = inspect(self.classes.Address)
-        path = PathRegistry.coerce((umapper, umapper.attrs.addresses,
-                                amapper, amapper.attrs.email_address))
+        path = PathRegistry.coerce(
+            (
+                umapper,
+                umapper.attrs.addresses,
+                amapper,
+                amapper.attrs.email_address,
+            )
+        )
         eq_(path[1:3], (umapper.attrs.addresses, amapper))
 
     def test_addition(self):
@@ -410,8 +548,14 @@ class PathRegistryTest(_fixtures.FixtureTest):
         p2 = PathRegistry.coerce((amapper, amapper.attrs.email_address))
         eq_(
             p1 + p2,
-            PathRegistry.coerce((umapper, umapper.attrs.addresses,
-                                amapper, amapper.attrs.email_address))
+            PathRegistry.coerce(
+                (
+                    umapper,
+                    umapper.attrs.addresses,
+                    amapper,
+                    amapper.attrs.email_address,
+                )
+            ),
         )
 
     def test_length(self):
@@ -421,8 +565,14 @@ class PathRegistryTest(_fixtures.FixtureTest):
         p0 = PathRegistry.coerce((umapper,))
         p1 = PathRegistry.coerce((umapper, umapper.attrs.addresses))
         p2 = PathRegistry.coerce((umapper, umapper.attrs.addresses, amapper))
-        p3 = PathRegistry.coerce((umapper, umapper.attrs.addresses,
-                                amapper, amapper.attrs.email_address))
+        p3 = PathRegistry.coerce(
+            (
+                umapper,
+                umapper.attrs.addresses,
+                amapper,
+                amapper.attrs.email_address,
+            )
+        )
 
         eq_(len(pneg1), 0)
         eq_(len(p0), 1)
@@ -444,11 +594,19 @@ class PathRegistryTest(_fixtures.FixtureTest):
         p3 = PathRegistry.coerce((umapper, umapper.attrs.name))
         p4 = PathRegistry.coerce((u_alias, umapper.attrs.addresses))
         p5 = PathRegistry.coerce((umapper, umapper.attrs.addresses, amapper))
-        p6 = PathRegistry.coerce((amapper, amapper.attrs.user, umapper,
-                                umapper.attrs.addresses))
-        p7 = PathRegistry.coerce((amapper, amapper.attrs.user, umapper,
-                                umapper.attrs.addresses,
-                                amapper, amapper.attrs.email_address))
+        p6 = PathRegistry.coerce(
+            (amapper, amapper.attrs.user, umapper, umapper.attrs.addresses)
+        )
+        p7 = PathRegistry.coerce(
+            (
+                amapper,
+                amapper.attrs.user,
+                umapper,
+                umapper.attrs.addresses,
+                amapper,
+                amapper.attrs.email_address,
+            )
+        )
 
         is_(p1 == p2, True)
         is_(p1 == p3, False)
@@ -477,15 +635,9 @@ class PathRegistryTest(_fixtures.FixtureTest):
         p2 = PathRegistry.coerce((umapper, umapper.attrs.addresses, amapper))
         p3 = PathRegistry.coerce((amapper, amapper.attrs.email_address))
 
-        eq_(
-            p1.path, (umapper, umapper.attrs.addresses)
-        )
-        eq_(
-            p2.path, (umapper, umapper.attrs.addresses, amapper)
-        )
-        eq_(
-            p3.path, (amapper, amapper.attrs.email_address)
-        )
+        eq_(p1.path, (umapper, umapper.attrs.addresses))
+        eq_(p2.path, (umapper, umapper.attrs.addresses, amapper))
+        eq_(p3.path, (amapper, amapper.attrs.email_address))
 
     def test_registry_set(self):
         reg = {}
@@ -502,10 +654,10 @@ class PathRegistryTest(_fixtures.FixtureTest):
         eq_(
             reg,
             {
-                ('p1key', p1.path): 'p1value',
-                ('p2key', p2.path): 'p2value',
-                ('p3key', p3.path): 'p3value',
-            }
+                ("p1key", p1.path): "p1value",
+                ("p2key", p2.path): "p2value",
+                ("p3key", p3.path): "p3value",
+            },
         )
 
     def test_registry_get(self):
@@ -518,9 +670,9 @@ class PathRegistryTest(_fixtures.FixtureTest):
         p3 = PathRegistry.coerce((amapper, amapper.attrs.email_address))
         reg.update(
             {
-                ('p1key', p1.path): 'p1value',
-                ('p2key', p2.path): 'p2value',
-                ('p3key', p3.path): 'p3value',
+                ("p1key", p1.path): "p1value",
+                ("p2key", p2.path): "p2value",
+                ("p3key", p3.path): "p3value",
             }
         )
 
@@ -540,9 +692,9 @@ class PathRegistryTest(_fixtures.FixtureTest):
         p3 = PathRegistry.coerce((amapper, amapper.attrs.email_address))
         reg.update(
             {
-                ('p1key', p1.path): 'p1value',
-                ('p2key', p2.path): 'p2value',
-                ('p3key', p3.path): 'p3value',
+                ("p1key", p1.path): "p1value",
+                ("p2key", p2.path): "p2value",
+                ("p3key", p3.path): "p3value",
             }
         )
         assert p1.contains(reg, "p1key")
@@ -557,11 +709,7 @@ class PathRegistryTest(_fixtures.FixtureTest):
 
         p1 = PathRegistry.coerce((umapper, umapper.attrs.addresses))
         p2 = PathRegistry.coerce((umapper, umapper.attrs.addresses, amapper))
-        reg.update(
-            {
-                ('p1key', p1.path): 'p1value',
-            }
-        )
+        reg.update({("p1key", p1.path): "p1value"})
 
         p1.setdefault(reg, "p1key", "p1newvalue_a")
         p1.setdefault(reg, "p1key_new", "p1newvalue_b")
@@ -569,10 +717,10 @@ class PathRegistryTest(_fixtures.FixtureTest):
         eq_(
             reg,
             {
-                ('p1key', p1.path): 'p1value',
-                ('p1key_new', p1.path): 'p1newvalue_b',
-                ('p2key', p2.path): 'p2newvalue',
-            }
+                ("p1key", p1.path): "p1value",
+                ("p1key_new", p1.path): "p1newvalue_b",
+                ("p2key", p2.path): "p2newvalue",
+            },
         )
 
     def test_serialize(self):
@@ -581,22 +729,19 @@ class PathRegistryTest(_fixtures.FixtureTest):
         umapper = inspect(self.classes.User)
         amapper = inspect(self.classes.Address)
 
-        p1 = PathRegistry.coerce((umapper, umapper.attrs.addresses, amapper,
-                            amapper.attrs.email_address))
+        p1 = PathRegistry.coerce(
+            (
+                umapper,
+                umapper.attrs.addresses,
+                amapper,
+                amapper.attrs.email_address,
+            )
+        )
         p2 = PathRegistry.coerce((umapper, umapper.attrs.addresses, amapper))
         p3 = PathRegistry.coerce((umapper, umapper.attrs.addresses))
-        eq_(
-            p1.serialize(),
-            [(User, "addresses"), (Address, "email_address")]
-        )
-        eq_(
-            p2.serialize(),
-            [(User, "addresses"), (Address, None)]
-        )
-        eq_(
-            p3.serialize(),
-            [(User, "addresses")]
-        )
+        eq_(p1.serialize(), [(User, "addresses"), (Address, "email_address")])
+        eq_(p2.serialize(), [(User, "addresses"), (Address, None)])
+        eq_(p3.serialize(), [(User, "addresses")])
 
     def test_deseralize(self):
         User = self.classes.User
@@ -604,29 +749,32 @@ class PathRegistryTest(_fixtures.FixtureTest):
         umapper = inspect(self.classes.User)
         amapper = inspect(self.classes.Address)
 
-
-        p1 = PathRegistry.coerce((umapper, umapper.attrs.addresses, amapper,
-                            amapper.attrs.email_address))
+        p1 = PathRegistry.coerce(
+            (
+                umapper,
+                umapper.attrs.addresses,
+                amapper,
+                amapper.attrs.email_address,
+            )
+        )
         p2 = PathRegistry.coerce((umapper, umapper.attrs.addresses, amapper))
         p3 = PathRegistry.coerce((umapper, umapper.attrs.addresses))
 
         eq_(
-            PathRegistry.deserialize([(User, "addresses"),
-                        (Address, "email_address")]),
-            p1
+            PathRegistry.deserialize(
+                [(User, "addresses"), (Address, "email_address")]
+            ),
+            p1,
         )
         eq_(
             PathRegistry.deserialize([(User, "addresses"), (Address, None)]),
-            p2
-        )
-        eq_(
-            PathRegistry.deserialize([(User, "addresses")]),
-            p3
+            p2,
         )
+        eq_(PathRegistry.deserialize([(User, "addresses")]), p3)
+
 
-from .inheritance import _poly_fixtures
 class PathRegistryInhTest(_poly_fixtures._Polymorphic):
-    run_setup_mappers = 'once'
+    run_setup_mappers = "once"
     run_inserts = None
     run_deletes = None
 
@@ -640,10 +788,7 @@ class PathRegistryInhTest(_poly_fixtures._Polymorphic):
 
         # given a mapper and an attribute on a subclass,
         # the path converts what you get to be against that subclass
-        eq_(
-            p1.path,
-            (emapper, emapper.attrs.machines)
-        )
+        eq_(p1.path, (emapper, emapper.attrs.machines))
 
     def test_plain_compound(self):
         Company = _poly_fixtures.Company
@@ -653,14 +798,20 @@ class PathRegistryInhTest(_poly_fixtures._Polymorphic):
         pmapper = inspect(Person)
         emapper = inspect(Engineer)
 
-        p1 = PathRegistry.coerce((cmapper, cmapper.attrs.employees,
-                        pmapper, emapper.attrs.machines))
+        p1 = PathRegistry.coerce(
+            (cmapper, cmapper.attrs.employees, pmapper, emapper.attrs.machines)
+        )
 
         # given a mapper and an attribute on a subclass,
         # the path converts what you get to be against that subclass
         eq_(
             p1.path,
-            (cmapper, cmapper.attrs.employees, emapper, emapper.attrs.machines)
+            (
+                cmapper,
+                cmapper.attrs.employees,
+                emapper,
+                emapper.attrs.machines,
+            ),
         )
 
     def test_plain_aliased(self):
@@ -674,10 +825,7 @@ class PathRegistryInhTest(_poly_fixtures._Polymorphic):
         p1 = PathRegistry.coerce((p_alias, emapper.attrs.machines))
         # plain AliasedClass - the path keeps that AliasedClass directly
         # as is in the path
-        eq_(
-            p1.path,
-            (p_alias, emapper.attrs.machines)
-        )
+        eq_(p1.path, (p_alias, emapper.attrs.machines))
 
     def test_plain_aliased_compound(self):
         Company = _poly_fixtures.Company
@@ -692,13 +840,19 @@ class PathRegistryInhTest(_poly_fixtures._Polymorphic):
         c_alias = inspect(c_alias)
         p_alias = inspect(p_alias)
 
-        p1 = PathRegistry.coerce((c_alias, cmapper.attrs.employees,
-                    p_alias, emapper.attrs.machines))
+        p1 = PathRegistry.coerce(
+            (c_alias, cmapper.attrs.employees, p_alias, emapper.attrs.machines)
+        )
         # plain AliasedClass - the path keeps that AliasedClass directly
         # as is in the path
         eq_(
             p1.path,
-            (c_alias, cmapper.attrs.employees, p_alias, emapper.attrs.machines)
+            (
+                c_alias,
+                cmapper.attrs.employees,
+                p_alias,
+                emapper.attrs.machines,
+            ),
         )
 
     def test_with_poly_sub(self):
@@ -714,10 +868,7 @@ class PathRegistryInhTest(_poly_fixtures._Polymorphic):
 
         # polymorphic AliasedClass - the path uses _entity_for_mapper()
         # to get the most specific sub-entity
-        eq_(
-            p1.path,
-            (e_poly, emapper.attrs.machines)
-        )
+        eq_(p1.path, (e_poly, emapper.attrs.machines))
 
     def test_with_poly_base(self):
         Person = _poly_fixtures.Person
@@ -733,10 +884,7 @@ class PathRegistryInhTest(_poly_fixtures._Polymorphic):
 
         # polymorphic AliasedClass - because "name" is on Person,
         # we get Person, not Engineer
-        eq_(
-            p1.path,
-            (p_poly, pmapper.attrs.name)
-        )
+        eq_(p1.path, (p_poly, pmapper.attrs.name))
 
     def test_with_poly_use_mapper(self):
         Person = _poly_fixtures.Person
@@ -750,8 +898,4 @@ class PathRegistryInhTest(_poly_fixtures._Polymorphic):
 
         # polymorphic AliasedClass with the "use_mapper_path" flag -
         # the AliasedClass acts just like the base mapper
-        eq_(
-            p1.path,
-            (emapper, emapper.attrs.machines)
-        )
-
+        eq_(p1.path, (emapper, emapper.attrs.machines))
